import { Query } from './types';
export declare enum FacetType {
    single = "single",
    multiple = "multiple"
}
/**
 * Interface for objects representing a facet.
 *
 * Currently, this interface only supports:
 *  - facets without a parameter for the optionsEndpoint
 *  - facets with only one parameter for the filterEndpoint
 *  - for facets with type=multiple: only disjunctive facets
 */
export interface Facet {
    optionsEndpoint: string;
    filterEndpoint: string;
    filterParameterName: string;
    filterParameterValue: string | undefined;
    type: FacetType;
    resetOnQueryChange: boolean;
}
/**
 * Associate Query objects with each other in a faceted search setup.
 */
export declare class FacetedSearch {
    private searchQuery;
    private emptyParameterQuery?;
    private modifiers?;
    private _facets;
    private _modifiers?;
    private _activeModifier?;
    constructor(searchQuery: Query, emptyParameterQuery?: Query | undefined, modifiers?: Query[] | undefined);
    /**
     * Add a facet to the FacetedSearch object.
     */
    addFacet(endpoint: string, type?: FacetType, resetOnQueryChange?: boolean, filterEndpointPostfix?: string, filterEndpointParameterName?: string): void;
    get facets(): Facet[];
    /**
     * Set a query as modifier. Only modifiers in the list passed to constructor are allowed.
     */
    setModifier(modifier: Query | undefined): void;
    /**
     * Get the currently active modifier query.
     */
    getModifier(): Query | undefined;
    /**
     * Get the Query to get the search results.
     * Will return searchQuery as passed to the constructor unless a emptyParameterQuery was
     * also passed and all parameters for searchQuery are empty.
     */
    getBaseQuery(): Query;
    /**
     * Get the Query objects to retrieve search results. This includes the facet Query, if applicable.
     */
    getResultsQuery(excludeModifier?: boolean): Query[];
    /**
     * Get the Query objects to retrieve the facet options. When using multiple facets, the facetEndpoint
     * parameter is required.
     */
    getFacetQuery(facetEndpoint: string, excludeModifier?: boolean): Query[];
    /**
     * Set a parameter value for the searchQuery
     */
    setParameter(name: string, value: string): void;
    /**
     * Clear all searchQuery parameters
     */
    clearParameters(): void;
    /**
     * Set the selected options for a given facet.
     */
    setFacetSelection(facetEndpoint: string, selection: string | string[]): void;
    /**
     * Clear the selection for a given facet.
     */
    clearFacetSelection(facetEndpoint?: string): void;
    setSearchQuery(query: Query): void;
}
