"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FacetedSearch = exports.FacetType = void 0;
var utils_1 = require("./utils");
var FacetType;
(function (FacetType) {
    FacetType["single"] = "single";
    FacetType["multiple"] = "multiple";
})(FacetType = exports.FacetType || (exports.FacetType = {}));
/**
 * Associate Query objects with each other in a faceted search setup.
 */
var FacetedSearch = /** @class */ (function () {
    function FacetedSearch(
    // Query object for the search results that will serve as the base for this facet.
    // The searchQuery is used to fetch the options of the facet and will be filtered once
    // one or more facet options are selected.
    // The searchQuery must have at least one parameter.
    searchQuery, 
    // emptyParameterQuery is used instead of searchQuery when the searchQuery parameters are all empty.
    // If no emptyParameterQuery is passed, searchQuery is fetched with empty parameters.
    emptyParameterQuery, modifiers) {
        this.searchQuery = searchQuery;
        this.emptyParameterQuery = emptyParameterQuery;
        this.modifiers = modifiers;
        // Internal list of Facet objects
        this._facets = [];
        // Throw an error if the searchQuery does not have parameters
        if (!searchQuery.parameters || Object.keys(searchQuery.parameters).length === 0) {
            throw new Error('searchQuery has no parameters. Please initialize with an empty parameter');
        }
        if (modifiers) {
            this._modifiers = modifiers;
        }
    }
    /**
     * Add a facet to the FacetedSearch object.
     */
    FacetedSearch.prototype.addFacet = function (endpoint, type, resetOnQueryChange, filterEndpointPostfix, filterEndpointParameterName) {
        if (type === void 0) { type = FacetType.single; }
        if (resetOnQueryChange === void 0) { resetOnQueryChange = true; }
        if (filterEndpointPostfix === void 0) { filterEndpointPostfix = ':FILTER'; }
        if (filterEndpointParameterName === void 0) { filterEndpointParameterName = 'value'; }
        this._facets.push({
            optionsEndpoint: endpoint,
            filterEndpoint: "".concat(endpoint).concat(filterEndpointPostfix),
            filterParameterName: filterEndpointParameterName,
            filterParameterValue: undefined,
            resetOnQueryChange: resetOnQueryChange,
            type: type,
        });
    };
    Object.defineProperty(FacetedSearch.prototype, "facets", {
        get: function () {
            return this._facets;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Set a query as modifier. Only modifiers in the list passed to constructor are allowed.
     */
    FacetedSearch.prototype.setModifier = function (modifier) {
        if (modifier === undefined) {
            this._activeModifier = undefined;
        }
        else if (this._modifiers) {
            var allowed = this._modifiers.find(function (m) { return m.endpoint === (modifier === null || modifier === void 0 ? void 0 : modifier.endpoint); });
            if (!allowed) {
                return;
            }
            this._activeModifier = modifier;
        }
    };
    /**
     * Get the currently active modifier query.
     */
    FacetedSearch.prototype.getModifier = function () {
        return this._activeModifier;
    };
    /**
     * Get the Query to get the search results.
     * Will return searchQuery as passed to the constructor unless a emptyParameterQuery was
     * also passed and all parameters for searchQuery are empty.
     */
    FacetedSearch.prototype.getBaseQuery = function () {
        if (this.emptyParameterQuery &&
            (!this.searchQuery.parameters ||
                Object.keys(this.searchQuery.parameters).length === 0 ||
                Object.values(this.searchQuery.parameters).every(function (p) { return !p || p === ''; }))) {
            return this.emptyParameterQuery;
        }
        else {
            return this.searchQuery;
        }
    };
    /**
     * Get the Query objects to retrieve search results. This includes the facet Query, if applicable.
     */
    FacetedSearch.prototype.getResultsQuery = function (excludeModifier) {
        if (excludeModifier === void 0) { excludeModifier = false; }
        var q = __spreadArray([
            this.getBaseQuery()
        ], __read(this._facets
            .filter(function (f) { return f.filterParameterValue !== undefined && f.filterParameterValue !== ''; })
            .map(function (f) {
            var _a;
            return ({
                endpoint: f.filterEndpoint,
                parameters: (_a = {}, _a[f.filterParameterName] = f.filterParameterValue, _a),
            });
        })), false);
        if (!excludeModifier && this._activeModifier !== undefined && this._activeModifier !== null) {
            q.push(this._activeModifier);
        }
        return q;
    };
    /**
     * Get the Query objects to retrieve the facet options. When using multiple facets, the facetEndpoint
     * parameter is required.
     */
    FacetedSearch.prototype.getFacetQuery = function (facetEndpoint, excludeModifier) {
        if (excludeModifier === void 0) { excludeModifier = false; }
        var facet = this._facets.find(function (f) { return f.optionsEndpoint === facetEndpoint; });
        if (!facet) {
            throw new Error('Facet not found in FacetedSearch');
        }
        var q = __spreadArray([
            this.getBaseQuery()
        ], __read(this._facets
            .filter(function (f) {
            return f.filterParameterValue !== undefined &&
                f.filterParameterValue !== '' &&
                f.optionsEndpoint !== facetEndpoint;
        })
            .map(function (f) {
            var _a;
            return ({
                endpoint: f.filterEndpoint,
                parameters: (_a = {}, _a[f.filterParameterName] = f.filterParameterValue, _a),
            });
        })), false);
        if (!excludeModifier && this._activeModifier !== undefined && this._activeModifier !== null) {
            q.push(this._activeModifier);
        }
        q.push({ endpoint: facet.optionsEndpoint });
        return q;
    };
    /**
     * Set a parameter value for the searchQuery
     */
    FacetedSearch.prototype.setParameter = function (name, value) {
        var _a;
        this.searchQuery = __assign(__assign({}, this.searchQuery), { parameters: __assign(__assign({}, this.searchQuery.parameters), (_a = {}, _a[name] = value, _a)) });
        this._facets.forEach(function (f) {
            if (f.resetOnQueryChange) {
                f.filterParameterValue = undefined;
            }
        });
    };
    /**
     * Clear all searchQuery parameters
     */
    FacetedSearch.prototype.clearParameters = function () {
        this.searchQuery = __assign(__assign({}, this.searchQuery), { parameters: Object.keys(this.searchQuery.parameters || {}).reduce(function (acc, cur) {
                var _a;
                return (__assign(__assign({}, acc), (_a = {}, _a[cur] = '', _a)));
            }, {}) });
    };
    /**
     * Set the selected options for a given facet.
     */
    FacetedSearch.prototype.setFacetSelection = function (facetEndpoint, selection) {
        if (!(selection instanceof Array)) {
            selection = [selection];
        }
        var facet = this._facets.find(function (f) { return f.optionsEndpoint === facetEndpoint; });
        if (!facet) {
            throw new Error("FacetedSearch does not contain facet ".concat(facetEndpoint));
        }
        if (selection.length === 0) {
            facet.filterParameterValue = undefined;
        }
        else if (facet.type === 'single') {
            if (selection.length > 1) {
                throw new Error("Facet ".concat(facetEndpoint, " is a single selection facet but more than one selected option was given."));
            }
            facet.filterParameterValue = selection[0];
        }
        else {
            facet.filterParameterValue = (0, utils_1.tupleListToString)(selection);
        }
    };
    /**
     * Clear the selection for a given facet.
     */
    FacetedSearch.prototype.clearFacetSelection = function (facetEndpoint) {
        if (facetEndpoint) {
            var facet = this._facets.find(function (f) { return f.optionsEndpoint === facetEndpoint; });
            if (!facet) {
                throw new Error("FacetedSearch does not contain facet ".concat(facetEndpoint));
            }
            facet.filterParameterValue = '';
        }
        else {
            this._facets.forEach(function (f) {
                f.filterParameterValue = '';
            });
        }
    };
    FacetedSearch.prototype.setSearchQuery = function (query) {
        this.searchQuery = query;
    };
    return FacetedSearch;
}());
exports.FacetedSearch = FacetedSearch;
//# sourceMappingURL=FacetedSearch.js.map