"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Authenticator = exports.DEFAULT_AUDIENCE = exports.DEFAULT_AUTH_SERVER = void 0;
exports.DEFAULT_AUTH_SERVER = 'https://login.spinque.com/';
exports.DEFAULT_AUDIENCE = 'https://rest.spinque.com/';
/**
 * Abstract class with utitily functions for working with access tokens such as storage
 */
var Authenticator = /** @class */ (function () {
    function Authenticator(_tokenCache) {
        var _this = this;
        this._tokenCache = _tokenCache;
        this._authInProgress = true;
        /**
         * A Promise that delays any operation until an access token is set (with intervals of 50ms)
         * This is used to delay incoming request from our app when an access token is already
         * being requested but has not yet been received.
         */
        this._waitForAccessToken = new Promise(function (resolve) {
            var wait = function () {
                setTimeout(function () { return (!_this._authInProgress && _this._accessToken ? resolve(_this._accessToken) : wait()); }, 50);
            };
            wait();
        });
        // First thing to do: check if there's an access token in localStorage
        var cachedToken = this._tokenCache.get();
        if (cachedToken) {
            // Set it as class property
            this._accessToken = cachedToken.accessToken;
            this._expires = cachedToken.expires;
            this._authInProgress = false;
        }
        else {
            this._authInProgress = false;
        }
    }
    Object.defineProperty(Authenticator.prototype, "accessToken", {
        /**
         * Promise that will resolve with an access token if available or undefined otherwise.
         * This will try to fetch a new access token if:
         *  - the class implements the fetchAccessToken method
         *  - no unexpired access token is stored in this class
         */
        get: function () {
            var _this = this;
            // If the class already stores an access token, return it
            if (this._accessToken && this._expires && this._expires > Date.now() + 1000) {
                return Promise.resolve(this._accessToken);
            }
            // If the class is already authenticating, wait for it
            if (this._authInProgress) {
                return this._waitForAccessToken;
            }
            this._authInProgress = true;
            return this.fetchAccessToken().then(function (res) {
                _this._authInProgress = false;
                if (res && 'accessToken' in res && 'expiresIn' in res) {
                    _this.setAccessToken(res.accessToken, res.expiresIn);
                    return _this._accessToken;
                }
                else {
                    return undefined;
                }
            });
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Puts an access token and the expiration time in storage for usage when calling Spinque Query API
     */
    Authenticator.prototype.setAccessToken = function (accessToken, expiresIn) {
        this._accessToken = accessToken;
        this._expires = Date.now() + expiresIn * 1000;
        this._tokenCache.set(this._accessToken, this._expires);
        this._authInProgress = false;
    };
    return Authenticator;
}());
exports.Authenticator = Authenticator;
//# sourceMappingURL=Authenticator.js.map