"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.splitIntoSentences = exports.truncate = exports.getSnippet = void 0;
var snowball = require("./snowball/snowball");
var dutch_1 = require("./snowball/stopwords/dutch");
var english_1 = require("./snowball/stopwords/english");
var DEFAULT_SIZE = 512;
var DEFAULT_HIGHLIGHT = false;
var DEFAULT_STOPWORDS = undefined;
var DEFAULT_STEMMER = undefined;
/**
 * Takes a piece of text and a text query and returns a snippet of the text that best matches the query.
 */
var getSnippet = function (text, query, options) {
    if (options === void 0) { options = {}; }
    // In case of empty text, return empty snippet
    if (!text) {
        return '';
    }
    // Prevent problems by cutting off extremely large texts
    var maxTextSize = Math.pow(2, 16);
    text = text.slice(0, maxTextSize);
    var size = options.size || DEFAULT_SIZE;
    // If there's no query, there's nothing to match on so just return the truncated text
    if (!query) {
        return (0, exports.truncate)(text, size);
    }
    var stemmer = options.stemmer || DEFAULT_STEMMER;
    var stopwords = options.stopwords || DEFAULT_STOPWORDS;
    var processor = new TextProcessor(stopwords, stemmer);
    var snippet = findBestSnippet(text, query, logWeightedTermFrequency, processor, size);
    if (options.highlight || DEFAULT_HIGHLIGHT) {
        return highlight(snippet, query, processor);
    }
    return snippet;
};
exports.getSnippet = getSnippet;
/**
 * Find the best snippet of a text for a query, given a scoring function and text processor.
 */
var findBestSnippet = function (text, query, score, processor, size) {
    var qterms = processor.tokenize(query);
    var sentences = (0, exports.splitIntoSentences)(text);
    var bestScore = -1;
    var bestSnippet = '';
    for (var i = 0; i < sentences.length; i++) {
        var snippet = '';
        var j = i;
        while (j < sentences.length && snippet.length < size) {
            snippet += ' ' + sentences[j];
            j++;
        }
        if (snippet.length > size) {
            var trimmed = snippet.slice(0, size);
            var lastSpace = trimmed.lastIndexOf(' ');
            snippet = trimmed.slice(0, lastSpace) + '...';
        }
        var snippetTerms = processor.tokenize(snippet);
        var _score = score(snippetTerms, qterms);
        if (_score > bestScore) {
            bestScore = _score;
            bestSnippet = snippet.trim();
        }
    }
    return bestSnippet;
};
/**
 * Highlight matching words by wrapping in <mark> tags
 */
var highlight = function (snippet, query, processor) {
    var qterms = processor.tokenize(query);
    return snippet.replace(/\b\w+\b/g, function (word) {
        var processed = processor.tokenize_one(word);
        if (processed !== null && qterms.includes(processed)) {
            return "<mark>".concat(word, "</mark>");
        }
        return word;
    });
};
/**
 * Truncates a piece of text to the desired maximum length and appends '...' to the end.
 */
var truncate = function (text, size) {
    if (size === void 0) { size = 512; }
    if (text.length <= size) {
        return text;
    }
    return "".concat(text.slice(0, size - 3), "...");
};
exports.truncate = truncate;
/**
 * Split text into sentence using a Regular Expression.
 */
var splitIntoSentences = function (text) {
    return text.match(/[^.!?]+[.!?]+/g) || [];
};
exports.splitIntoSentences = splitIntoSentences;
/**
 * Scoring function to rank snippets by. Plain term frequency over all query terms
 */
var termFrequency = function (docterms, qterms) {
    var score = 0;
    qterms.forEach(function (qterm) {
        score += docterms.filter(function (s) { return s === qterm; }).length;
    });
    return score;
};
/**
 * Scoring function to rank snippets by. Takes the log of term frequency per distinct query term.
 */
var logWeightedTermFrequency = function (docterms, qterms) {
    var score = 0;
    qterms.forEach(function (qterm) {
        score += Math.log(1 + docterms.filter(function (s) { return s === qterm; }).length);
    });
    return score;
};
// Dictionary of stopword lists per language
var STOPWORDS_PER_LANG = { dutch: dutch_1.stopwords, english: english_1.stopwords };
/**
 * Cache the stemmer in memory to prevent calling `newStemmer` often.
 */
var cachedStemmers = {};
var getStemmer = function (stemmer) {
    if (!cachedStemmers[stemmer]) {
        cachedStemmers[stemmer] = snowball.newStemmer(stemmer);
    }
    return cachedStemmers[stemmer];
};
var TextProcessor = /** @class */ (function () {
    function TextProcessor(stopwords, stemmer) {
        if (stopwords) {
            this.stopwords = STOPWORDS_PER_LANG[stopwords];
        }
        else {
            this.stopwords = new Set();
        }
        if (stemmer) {
            this.stemmer = getStemmer(stemmer).stem;
        }
        else {
            this.stemmer = function (word) { return word; };
        }
    }
    TextProcessor.prototype.tokenize = function (text) {
        var _this = this;
        // split on word boundaries to get list of words
        var words = text.match(/\b\w+\b/g) || [];
        return words.reduce(function (acc, word) {
            var processed = _this.tokenize_one(word);
            return processed ? __spreadArray(__spreadArray([], __read(acc), false), [processed], false) : acc;
        }, []);
    };
    /**
     * Processed one word by checking the list of stopwords and stemming
     * @param word
     * @returns null if the word should be ignored
     */
    TextProcessor.prototype.tokenize_one = function (word) {
        word = word.toLowerCase();
        // Ignore if the word is in the list of stopwords
        if (this.stopwords.has(word)) {
            return null;
        }
        // Return the list with the new word
        return this.stemmer(word);
    };
    return TextProcessor;
}());
//# sourceMappingURL=snippet.js.map