"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("../utils");
describe('utils', function () {
    it('pathFromQuery should convert a Query to a string', function () {
        var q1 = { endpoint: 'my-endpoint' };
        expect(typeof (0, utils_1.pathFromQuery)(q1)).toBe('string');
    });
    it('pathFromQuery should convert a Query to a path that does not start with a slash', function () {
        var q1 = { endpoint: 'my-endpoint' };
        expect((0, utils_1.pathFromQuery)(q1)[0]).not.toBe('/');
    });
    it('pathFromQuery should convert a Query without parameter to a simple path starting with e', function () {
        var q1 = { endpoint: 'my-endpoint' };
        expect((0, utils_1.pathFromQuery)(q1)).toBe('e/my-endpoint');
    });
    it('pathFromQuery should use URL encoding in endpoint names', function () {
        var q1 = { endpoint: 'my-"crazy$-endpoint' };
        expect((0, utils_1.pathFromQuery)(q1)).toBe('e/my-%22crazy%24-endpoint');
    });
    it('pathFromQuery should add parameters to the path', function () {
        var q1 = { endpoint: 'my-endpoint', parameters: { 'my-param': 'my-value' } };
        expect((0, utils_1.pathFromQuery)(q1)).toBe('e/my-endpoint/p/my-param/my-value');
    });
    it('pathFromQuery should use URL encoding in parameter names', function () {
        var q1 = { endpoint: 'my-endpoint', parameters: { 'my-"crazy$-param': 'my-value' } };
        expect((0, utils_1.pathFromQuery)(q1)).toBe('e/my-endpoint/p/my-%22crazy%24-param/my-value');
    });
    it('pathFromQuery should use URL encoding in parameter values', function () {
        var q1 = { endpoint: 'my-endpoint', parameters: { 'my-param': 'my-"crazy$-value' } };
        expect((0, utils_1.pathFromQuery)(q1)).toBe('e/my-endpoint/p/my-param/my-%22crazy%24-value');
    });
    it('pathFromQuery should add multiple parameters to the path', function () {
        var q1 = {
            endpoint: 'my-endpoint',
            parameters: { one: 'value-one', two: 'value-two', three: 'value-three' },
        };
        expect((0, utils_1.pathFromQuery)(q1)).toBe('e/my-endpoint/p/one/value-one/p/two/value-two/p/three/value-three');
    });
    it('pathFromQueries should work the same as pathFromQuery for a single Query', function () {
        var q1 = {
            endpoint: 'my-endpoint',
            parameters: { one: 'value-one', two: 'value-two', three: 'value-three' },
        };
        expect((0, utils_1.pathFromQuery)(q1)).toEqual((0, utils_1.pathFromQueries)([q1]));
        var q2 = { endpoint: 'my-"crazy$-endpoint' };
        expect((0, utils_1.pathFromQuery)(q2)).toEqual((0, utils_1.pathFromQueries)([q2]));
    });
    it('pathFromQueries should join multiple Queries together', function () {
        var q1 = {
            endpoint: 'my-endpoint',
            parameters: { one: 'value-one', two: 'value-two', three: 'value-three' },
        };
        var q2 = { endpoint: 'my-"crazy$-endpoint' };
        expect((0, utils_1.pathFromQueries)([q1, q2])).toEqual((0, utils_1.pathFromQuery)(q1) + '/' + (0, utils_1.pathFromQuery)(q2));
        expect((0, utils_1.pathFromQueries)([q1, q2])).toEqual('e/my-endpoint/p/one/value-one/p/two/value-two/p/three/value-three/e/my-%22crazy%24-endpoint');
        expect((0, utils_1.pathFromQueries)([q2, q1])).toEqual((0, utils_1.pathFromQuery)(q2) + '/' + (0, utils_1.pathFromQuery)(q1));
    });
    it('urlFromQueries should prepend pathFromQueries with the base URL, version and workspace', function () {
        var apiConfig = {
            baseUrl: 'https://rest.spinque.com/',
            version: '4',
            workspace: 'my-workspace',
            api: 'my-api',
            config: 'my-config',
        };
        var query = { endpoint: 'my-endpoint' };
        expect((0, utils_1.urlFromQueries)(apiConfig, query)).toEqual('https://rest.spinque.com/4/my-workspace/api/my-api/e/my-endpoint/results?config=my-config');
    });
    it('urlFromQueries should add count and offset parameters', function () {
        var apiConfig = {
            baseUrl: 'https://rest.spinque.com/',
            version: '4',
            workspace: 'my-workspace',
            api: 'my-api',
            config: 'my-config',
        };
        var query = { endpoint: 'my-endpoint' };
        expect((0, utils_1.urlFromQueries)(apiConfig, query, { count: 314, offset: 15 })).toEqual('https://rest.spinque.com/4/my-workspace/api/my-api/e/my-endpoint/results?config=my-config&count=314&offset=15');
    });
    it('urlFromQueries should fail without a baseUrl, version, workspace or API name', function () {
        var query = { endpoint: 'my-endpoint' };
        var apiConfig1 = {
            version: '4',
            workspace: 'my-workspace',
            api: 'my-api',
            config: 'my-config',
        };
        expect(function () { return (0, utils_1.urlFromQueries)(apiConfig1, query); }).toThrow('Base URL missing');
        var apiConfig2 = {
            baseUrl: 'https://rest.spinque.com/',
            workspace: 'my-workspace',
            api: 'my-api',
            config: 'my-config',
        };
        expect(function () { return (0, utils_1.urlFromQueries)(apiConfig2, query); }).toThrow('Version missing');
        var apiConfig3 = {
            baseUrl: 'https://rest.spinque.com/',
            version: '4',
            api: 'my-api',
            config: 'my-config',
        };
        expect(function () { return (0, utils_1.urlFromQueries)(apiConfig3, query); }).toThrow('Workspace missing');
        var apiConfig4 = {
            baseUrl: 'https://rest.spinque.com/',
            version: '4',
            workspace: 'my-workspace',
            config: 'my-config',
        };
        expect(function () { return (0, utils_1.urlFromQueries)(apiConfig4, query); }).toThrow('API name missing');
    });
    it('urlFromQueries should not fail without config', function () {
        var apiConfig = {
            baseUrl: 'https://rest.spinque.com/',
            version: '4',
            workspace: 'my-workspace',
            api: 'my-api',
        };
        var query = { endpoint: 'my-endpoint' };
        expect((0, utils_1.urlFromQueries)(apiConfig, query)).toEqual('https://rest.spinque.com/4/my-workspace/api/my-api/e/my-endpoint/results');
    });
    it('urlFromQueries should be able to request statistics', function () {
        var apiConfig = {
            baseUrl: 'https://rest.spinque.com/',
            version: '4',
            workspace: 'my-workspace',
            api: 'my-api',
            config: 'default',
        };
        var query = { endpoint: 'my-endpoint' };
        expect((0, utils_1.urlFromQueries)(apiConfig, query, {}, 'statistics')).toEqual('https://rest.spinque.com/4/my-workspace/api/my-api/e/my-endpoint/statistics?config=default');
    });
    it('tupleListToString should convert everything to strings', function () {
        expect(typeof (0, utils_1.tupleListToString)(0)).toEqual('string');
        expect(typeof (0, utils_1.tupleListToString)('hello')).toEqual('string');
    });
    it('tupleListToString should convert single strings or numbers', function () {
        expect((0, utils_1.tupleListToString)('hello')).toEqual('1(hello)');
        expect((0, utils_1.tupleListToString)(123)).toEqual('1(123)');
    });
    it('tupleListToString should convert an array of strings or numbers', function () {
        expect((0, utils_1.tupleListToString)(['hello'])).toEqual('1(hello)');
        expect((0, utils_1.tupleListToString)(['hello', 'there'])).toEqual('1(hello)|1(there)');
        expect((0, utils_1.tupleListToString)([123])).toEqual('1(123)');
        expect((0, utils_1.tupleListToString)([123, 456])).toEqual('1(123)|1(456)');
        expect((0, utils_1.tupleListToString)([123, 'marco'])).toEqual('1(123)|1(marco)');
        expect((0, utils_1.tupleListToString)(['polo', '456'])).toEqual('1(polo)|1(456)');
    });
    it('tupleListToString should convert an array of arrays of strings or numbers', function () {
        expect((0, utils_1.tupleListToString)([
            ['hello', 'there'],
            ['marco', 'polo'],
            [123, 456],
        ])).toEqual('1(hello,there)|1(marco,polo)|1(123,456)');
    });
    it('tupleListToString should throw an error if arrays are not of equal length', function () {
        expect(function () { return (0, utils_1.tupleListToString)([['hello', 'there'], ['marco'], [123, 456]]); }).toThrow();
    });
    it('tupleListToString should use scores if passed', function () {
        expect((0, utils_1.tupleListToString)([
            ['hello', 'there'],
            ['marco', 'polo'],
            [123, 456],
        ], [0.1, 0.5, 3.1415])).toEqual('0.1(hello,there)|0.5(marco,polo)|3.1415(123,456)');
    });
    it('tupleListToString should throw an error if scores length does not match arrays length', function () {
        expect(function () { return (0, utils_1.tupleListToString)([['hello', 'there'], ['marco'], [123, 456]], []); }).toThrow();
        expect(function () { return (0, utils_1.tupleListToString)([['hello', 'there'], ['marco'], [123, 456]], [1]); }).toThrow();
        expect(function () { return (0, utils_1.tupleListToString)([['hello', 'there'], ['marco'], [123, 456]], [1, 2]); }).toThrow();
        expect(function () { return (0, utils_1.tupleListToString)([['hello', 'there'], ['marco'], [123, 456]], [1, 2, 3, 4]); }).toThrow();
    });
});
//# sourceMappingURL=utils.spec.js.map