import { TokenCache } from './authentication';
/**
 * Configuration of an API to send queries to. Used to instantiate the Api class.
 */
export interface ApiConfig {
    /**
     * URL to the Spinque Query API deployment.
     *
     * @default https://rest.spinque.com/
     */
    baseUrl?: string;
    /**
     * Version of the Spinque Query API deployment.
     *
     * @default 4
     */
    version?: string;
    /**
     * Name of the Spinque workspace that should be addressed.
     * The Spinque Desk administrator working on your project knowns this value.
     */
    workspace?: string;
    /**
     * Name of the API that is provided by the workspace.
     * The Spinque Desk administrator working on your project knowns this value.
     */
    api?: string;
    /**
     * Name of the configuration of the Spinque workspace that should be used.
     * Usually, this is something like 'production', 'development' or 'default'.
     * The Spinque Desk administrator working on your project knowns this value.
     *
     * @default default
     */
    config?: string;
    /**
     * Authentication that should be used when querying the API.
     */
    authentication?: ApiAuthenticationConfig;
}
/**
 * Authentication configuration for an API.
 */
export declare type ApiAuthenticationConfig = {
    /**
     * OAuth 2.0 Client Credentials flow.
     * Uses Client ID and Client Secret to directly request an access token.
     *
     * @see https://oauth.net/2/grant-types/client-credentials/
     */
    type: 'client-credentials';
    /**
     * URL to the authentication server to use.
     *
     * @default https://login.spinque.com/
     */
    authServer?: string;
    clientId: string;
    clientSecret: string;
    /**
     * Implementation of a TokenCache. Defines a get and set method that put the token in some sort of storage.
     * This is especially useful during development, when in-memory caching doesn't work due to frequent server restarts.
     */
    tokenCache?: TokenCache;
} | {
    /**
     * OAuth 2.0 PKCE flow.
     *
     * @see https://oauth.net/2/pkce/
     */
    type: 'pkce';
    /**
     * URL to the authentication server to use.
     *
     * @default https://login.spinque.com/
     */
    authServer?: string;
    clientId: string;
    callback: string;
    /**
     * Implementation of a TokenCache. Defines a get and set method that put the token in some sort of storage.
     * This is especially useful during development, when in-memory caching doesn't work due to frequent server restarts.
     */
    tokenCache?: TokenCache;
};
/**
 * Represent a single query, consisting of an endpoint name and a map of parameters (name and value).
 */
export interface Query {
    /**
     * Name of the endpoint.
     */
    endpoint: string;
    /**
     * Map of filled parameters (name and value).
     */
    parameters?: {
        [name: string]: string;
    };
}
/**
 * Any request to the Spinque Query API must be one of these types.
 * 'results' will return actual result items.
 * 'statistics' will return the probability distribution for the query.
 */
export declare type RequestType = 'results' | 'statistics' | 'count' | 'results,count';
export interface RequestOptions {
    count?: number;
    offset?: number;
    format?: 'json' | 'xml' | 'rdf' | 'csv' | 'xlsx';
    level?: number;
    homogeneousArrays?: boolean;
}
/**
 * Map from RequestType to a Response type
 */
declare type ResponseMap<T = ResultItemTupleTypes[]> = {
    results: ResultsResponse<T>;
    statistics: StatisticsResponse;
    count: CountResponse;
    'results,count': ResultsAndCountResponse<T>;
};
/**
 * ResponseType based on RequestType
 */
export declare type ResponseType<U extends RequestType, T = ResultItemTupleTypes[]> = U extends keyof ResponseMap ? ResponseMap<T>[U] : never;
/**
 * Data types known by Spinque.
 * OBJ is an object with attributes.
 * TUPLE_LIST is a list of tuples.
 * STRING, DATE, INTEGER, DOUBLE are simple data types.
 */
export declare type DataType = 'OBJ' | 'STRING' | 'DATE' | 'INTEGER' | 'DOUBLE' | 'TUPLE_LIST';
export declare type ResultItemTupleTypes = string | number | SpinqueResultObject;
/**
 * Response to a Query that contains the results
 */
export interface ResultsResponse<T = ResultItemTupleTypes[]> {
    count: number;
    offset: number;
    type: DataType[];
    items: ResultItem<T>[];
}
export interface ResultItem<T = ResultItemTupleTypes[]> {
    rank: number;
    probability: number;
    tuple: T;
}
/**
 * Output format of Spinque's OBJ type
 */
export interface SpinqueResultObject {
    /**
     * The unique identifier of the object.
     * Will always be a valid URI.
     */
    id: string;
    /**
     * Array of classes assigned to this object.
     *
     * If a value is in this list, then there is
     * a <https://www.w3.org/2000/01/rdf-schema#Class>
     * relation between that value and `id`.
     */
    class: string[];
    /**
     * Object with attribute names and values.
     * Values can be any valid JSON type.
     */
    attributes?: {
        [attributeName: string]: any;
    };
}
/**
 * Response to a Query that contains statistics
 */
export interface StatisticsResponse {
    total: number;
    stats: {
        cutoff: string;
        numResults: number;
    }[];
}
/**
 * Response to a Query that contains the count
 */
export interface CountResponse {
    total: number;
}
export declare type ResultsAndCountResponse<T = ResultItemTupleTypes[]> = [ResultsResponse<T>, StatisticsResponse];
/**
 * Generic error response class. Is implemented by more specific error type classes.
 */
export declare class ErrorResponse {
    message: string;
    status: number;
    constructor(message: string, status: number);
}
/**
 * Error class used when Spinque cannot find the workspace configuration you requested.
 * The workspace or configuration might be misspelled or removed.
 */
export declare class WorkspaceConfigNotFoundError implements ErrorResponse {
    message: string;
    status: number;
    constructor(message: string, status: number);
}
/**
 * Error class used when Spinque cannot find the API you requested.
 * The API might be misspelled or removed.
 */
export declare class ApiNotFoundError implements ErrorResponse {
    message: string;
    status: number;
    constructor(message: string, status: number);
}
/**
 * Error class used when Spinque cannot find the endpoint you requested.
 * The endpoint might be misspelled or removed.
 */
export declare class EndpointNotFoundError implements ErrorResponse {
    message: string;
    status: number;
    constructor(message: string, status: number);
}
/**
 * Error class used when you are not authorized to request results for
 * this workspace, API or endpoint.
 */
export declare class UnauthorizedError implements ErrorResponse {
    message: string;
    status: number;
    constructor(message: string, status: number);
}
/**
 * Error class when something fails on the side of Spinque. Please contact
 * your system administrator when this happens.
 */
export declare class ServerError implements ErrorResponse {
    message: string;
    status: number;
    constructor(message: string, status: number);
}
export {};
